@@ -27,6 +27,7 @@
 #include "lib/jxl/color_encoding_internal.h"
 #include "lib/jxl/color_management.h"
 #include "lib/jxl/common.h"
+#include "lib/jxl/enc_image_bundle.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/image_bundle.h"
 #include "lib/jxl/image_ops.h"
@@ -72,7 +73,7 @@ bool MarkerIsExif(const jpeg_saved_marker_ptr marker) {
 }
 
 Status ReadICCProfile(jpeg_decompress_struct* const cinfo,
-                      PaddedBytes* const icc) {
+                      std::vector<uint8_t>* const icc) {
   constexpr size_t kICCSignatureSize = sizeof kICCSignature;
   // ICC signature + uint8_t index + uint8_t max_index.
   constexpr size_t kICCHeadSize = kICCSignatureSize + 2;
@@ -149,7 +150,8 @@ Status ReadICCProfile(jpeg_decompress_struct* const cinfo,
   return true;
 }
 
-void ReadExif(jpeg_decompress_struct* const cinfo, PaddedBytes* const exif) {
+void ReadExif(jpeg_decompress_struct* const cinfo,
+              std::vector<uint8_t>* const exif) {
   constexpr size_t kExifSignatureSize = sizeof kExifSignature;
   for (jpeg_saved_marker_ptr marker = cinfo->marker_list; marker != nullptr;
        marker = marker->next) {
@@ -247,9 +249,9 @@ Status DecodeImageJPGCoefficients(Span<const uint8_t> bytes, CodecInOut* io) {
 }
 
 Status DecodeImageJPG(const Span<const uint8_t> bytes,
-                      const ColorHints& color_hints, ThreadPool* pool,
-                      CodecInOut* io, double* const elapsed_deinterleave) {
-  if (elapsed_deinterleave != nullptr) *elapsed_deinterleave = 0;
+                      const ColorHints& color_hints,
+                      const SizeConstraints& constraints,
+                      PackedPixelFile* ppf) {
   // Don't do anything for non-JPEG files (no need to report an error)
   if (!IsJPG(bytes)) return false;
 
@@ -262,7 +264,6 @@ Status DecodeImageJPG(const Span<const uint8_t> bytes,
   PaddedBytes icc;
   Image3F image;
   std::unique_ptr<JSAMPLE[]> row;
-  ImageBundle bundle(&io->metadata.m);
 
   const auto try_catch_block = [&]() -> bool {
     jpeg_decompress_struct cinfo;
@@ -292,7 +293,7 @@ Status DecodeImageJPG(const Span<const uint8_t> bytes,
       jpeg_destroy_decompress(&cinfo);
       return JXL_FAILURE("%s", str);
     };
-    if (!VerifyDimensions(&io->constraints, cinfo.image_width,
+    if (!VerifyDimensions(&constraints, cinfo.image_width,
                           cinfo.image_height)) {
       return failure("image too big");
     }
@@ -300,59 +301,69 @@ Status DecodeImageJPG(const Span<const uint8_t> bytes,
     if (cinfo.arith_code) {
       return failure("arithmetic code JPEGs are not supported");
     }
-    if (ReadICCProfile(&cinfo, &icc)) {
-      if (!color_encoding.SetICC(std::move(icc))) {
-        return failure("read an invalid ICC profile");
-      }
-    } else {
-      color_encoding = ColorEncoding::SRGB(cinfo.output_components == 1);
+    if (!ReadICCProfile(&cinfo, &ppf->icc)) {
+      ppf->icc.clear();
+      // Default to SRGB
+      ppf->color_encoding.color_space = cinfo.output_components == 1
+                                            ? JXL_COLOR_SPACE_GRAY
+                                            : JXL_COLOR_SPACE_RGB;
+      ppf->color_encoding.white_point = JXL_WHITE_POINT_D65;
+      ppf->color_encoding.primaries = JXL_PRIMARIES_SRGB;
+      ppf->color_encoding.transfer_function = JXL_TRANSFER_FUNCTION_SRGB;
+      ppf->color_encoding.rendering_intent = JXL_RENDERING_INTENT_PERCEPTUAL;
     }
-    ReadExif(&cinfo, &io->blobs.exif);
-    io->metadata.m.SetUintSamples(BITS_IN_JSAMPLE);
-    io->metadata.m.color_encoding = color_encoding;
+    ReadExif(&cinfo, &ppf->metadata.exif);
     int nbcomp = cinfo.num_components;
     if (nbcomp != 1 && nbcomp != 3) {
       return failure("unsupported number of components in JPEG");
     }
-    if (!ApplyColorHints(color_hints, /*color_already_set=*/true, false, io)) {
+    if (!ApplyColorHints(color_hints, /*color_already_set=*/true,
+                         /*is_gray=*/false, ppf)) {
       return failure("ApplyColorHints failed");
     }
 
+    ppf->info.xsize = cinfo.image_width;
+    ppf->info.ysize = cinfo.image_height;
+    // Original data is uint, so exponent_bits_per_sample = 0.
+    ppf->info.bits_per_sample = BITS_IN_JSAMPLE;
+    JXL_ASSERT(BITS_IN_JSAMPLE == 8 || BITS_IN_JSAMPLE == 16);
+    ppf->info.exponent_bits_per_sample = 0;
+    ppf->info.uses_original_profile = true;
+
+    // No alpha in JPG
+    ppf->info.alpha_bits = 0;
+    ppf->info.alpha_exponent_bits = 0;
+
+    ppf->info.num_color_channels = nbcomp;
+    ppf->info.orientation = JXL_ORIENT_IDENTITY;
+
     jpeg_start_decompress(&cinfo);
     JXL_ASSERT(cinfo.output_components == nbcomp);
-    image = Image3F(cinfo.image_width, cinfo.image_height);
-    row.reset(new JSAMPLE[cinfo.output_components * cinfo.image_width]);
-    for (size_t y = 0; y < image.ysize(); ++y) {
-      JSAMPROW rows[] = {row.get()};
+
+    const JxlPixelFormat format{
+        /*num_channels=*/static_cast<uint32_t>(nbcomp),
+        /*data_type=*/BITS_IN_JSAMPLE == 8 ? JXL_TYPE_UINT8 : JXL_TYPE_UINT16,
+        /*endianness=*/JXL_NATIVE_ENDIAN,
+        /*align=*/0,
+    };
+    ppf->frames.clear();
+    // Allocates the frame buffer.
+    ppf->frames.emplace_back(cinfo.image_width, cinfo.image_height, format);
+    const auto& frame = ppf->frames.back();
+    JXL_ASSERT(sizeof(JSAMPLE) * cinfo.output_components * cinfo.image_width <=
+               frame.color.stride);
+
+    for (size_t y = 0; y < cinfo.image_height; ++y) {
+      JSAMPROW rows[] = {reinterpret_cast<JSAMPLE*>(
+          static_cast<uint8_t*>(frame.color.pixels()) +
+          frame.color.stride * y)};
       jpeg_read_scanlines(&cinfo, rows, 1);
-      msan::UnpoisonMemory(
-          row.get(),
-          sizeof(JSAMPLE) * cinfo.output_components * cinfo.image_width);
-      auto start = Now();
-      float* const JXL_RESTRICT output_row[] = {
-          image.PlaneRow(0, y), image.PlaneRow(1, y), image.PlaneRow(2, y)};
-      if (cinfo.output_components == 1) {
-        for (size_t x = 0; x < image.xsize(); ++x) {
-          output_row[0][x] = output_row[1][x] = output_row[2][x] =
-              row[x] * (1.f / kJPEGSampleMultiplier);
-        }
-      } else {  // 3 components
-        for (size_t x = 0; x < image.xsize(); ++x) {
-          for (size_t c = 0; c < 3; ++c) {
-            output_row[c][x] = row[3 * x + c] * (1.f / kJPEGSampleMultiplier);
-          }
-        }
-      }
-      auto end = Now();
-      if (elapsed_deinterleave != nullptr) {
-        *elapsed_deinterleave += end - start;
-      }
+      msan::UnpoisonMemory(rows[0], sizeof(JSAMPLE) * cinfo.output_components *
+                                        cinfo.image_width);
     }
-    io->SetFromImage(std::move(image), color_encoding);
 
     jpeg_finish_decompress(&cinfo);
     jpeg_destroy_decompress(&cinfo);
-    io->dec_pixels = io->xsize() * io->ysize();
     return true;
   };
 
