@@ -12,6 +12,7 @@
 #include <stdint.h>
 
 #include "lib/extras/color_hints.h"
+#include "lib/extras/packed_image.h"
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/padded_bytes.h"
 #include "lib/jxl/base/span.h"
@@ -24,8 +25,8 @@ namespace extras {
 
 // Decodes `bytes` into `io`.
 Status DecodeImagePGX(const Span<const uint8_t> bytes,
-                      const ColorHints& color_hints, ThreadPool* pool,
-                      CodecInOut* io);
+                      const ColorHints& color_hints,
+                      const SizeConstraints& constraints, PackedPixelFile* ppf);
 
 // Transforms from io->c_current to `c_desired` and encodes into `bytes`.
 Status EncodeImagePGX(const CodecInOut* io, const ColorEncoding& c_desired,
