@@ -22,6 +22,7 @@
 #include "lib/jxl/base/byte_order.h"
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/file_io.h"
+#include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/color_management.h"
 #include "lib/jxl/common.h"
 #include "lib/jxl/dec_external_image.h"
@@ -69,7 +70,8 @@ class BlobsReaderPNG {
       }
       if (type == "exif") {
         if (!metadata->exif.empty()) {
-          JXL_WARNING("overwriting EXIF (%zu bytes) with base16 (%zu bytes)",
+          JXL_WARNING("overwriting EXIF (%" PRIuS " bytes) with base16 (%" PRIuS
+                      " bytes)",
                       metadata->exif.size(), bytes.size());
         }
         metadata->exif = std::move(bytes);
@@ -79,14 +81,15 @@ class BlobsReaderPNG {
         // TODO (jon): Deal with 8bim in some way
       } else if (type == "xmp") {
         if (!metadata->xmp.empty()) {
-          JXL_WARNING("overwriting XMP (%zu bytes) with base16 (%zu bytes)",
+          JXL_WARNING("overwriting XMP (%" PRIuS " bytes) with base16 (%" PRIuS
+                      " bytes)",
                       metadata->xmp.size(), bytes.size());
         }
         metadata->xmp = std::move(bytes);
       } else {
-        JXL_WARNING(
-            "Unknown type in 'Raw format type' text chunk: %s: %zu bytes",
-            type.c_str(), bytes.size());
+        JXL_WARNING("Unknown type in 'Raw format type' text chunk: %s: %" PRIuS
+                    " bytes",
+                    type.c_str(), bytes.size());
       }
     }
 
@@ -234,7 +237,8 @@ class BlobsWriterPNG {
     snprintf(key, sizeof(key), "Raw profile type %s", type.c_str());
 
     char header[30];
-    snprintf(header, sizeof(header), "\n%s\n%8zu", type.c_str(), bytes.size());
+    snprintf(header, sizeof(header), "\n%s\n%8" PRIuS, type.c_str(),
+             bytes.size());
 
     const std::string& encoded = std::string(header) + base16;
     if (lodepng_add_text(info, key, encoded.c_str()) != 0) {
