@@ -31,12 +31,13 @@
 
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/data_parallel.h"
+#include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/field_encodings.h"
 #include "lib/jxl/linalg.h"
 #include "lib/jxl/transfer_functions-inl.h"
 #if JPEGXL_ENABLE_SKCMS
-#include "skcms.h"
+#include "lib/jxl/enc_jxl_skcms.h"
 #else  // JPEGXL_ENABLE_SKCMS
 #include "lcms2.h"
 #include "lcms2_plugin.h"
@@ -310,7 +311,7 @@ Status CreateProfileXYZ(const cmsContext context,
 // IMPORTANT: icc must outlive profile.
 Status DecodeProfile(const PaddedBytes& icc, skcms_ICCProfile* const profile) {
   if (!skcms_Parse(icc.data(), icc.size(), profile)) {
-    return JXL_FAILURE("Failed to parse ICC profile with %zu bytes",
+    return JXL_FAILURE("Failed to parse ICC profile with %" PRIuS " bytes",
                        icc.size());
   }
   return true;
@@ -845,7 +846,8 @@ Status ColorSpaceTransform::Init(const ColorEncoding& c_src,
   const size_t channels_dst = c_dst.Channels();
   JXL_CHECK(channels_src == channels_dst);
 #if JXL_CMS_VERBOSE
-  printf("Channels: %zu; Threads: %zu\n", channels_src, num_threads);
+  printf("Channels: %" PRIuS "; Threads: %" PRIuS "\n", channels_src,
+         num_threads);
 #endif
 
 #if !JPEGXL_ENABLE_SKCMS
