@@ -213,7 +213,8 @@ uint64_t FrameFlagsFromParams(const CompressParams& cparams) {
   // We don't add noise at low butteraugli distances because the original
   // noise is stored within the compressed image and adding noise makes things
   // worse.
-  if (ApplyOverride(cparams.noise, dist >= kMinButteraugliForNoise)) {
+  if (ApplyOverride(cparams.noise, dist >= kMinButteraugliForNoise) ||
+      cparams.photon_noise_iso > 0) {
     flags |= FrameHeader::kNoise;
   }
 
@@ -793,10 +794,10 @@ class LossyFrameEncoder {
 
     auto& dct = enc_state_->shared.block_ctx_map.dc_thresholds;
     auto& num_dc_ctxs = enc_state_->shared.block_ctx_map.num_dc_ctxs;
-    enc_state_->shared.block_ctx_map.num_dc_ctxs = 1;
+    num_dc_ctxs = 1;
     for (size_t i = 0; i < 3; i++) {
       dct[i].clear();
-      int num_thresholds = (CeilLog2Nonzero(total_dc[i]) - 10) / 2;
+      int num_thresholds = (CeilLog2Nonzero(total_dc[i]) - 12) / 2;
       // up to 3 buckets per channel:
       // dark/medium/bright, yellow/unsat/blue, green/unsat/red
       num_thresholds = std::min(std::max(num_thresholds, 0), 2);
@@ -822,9 +823,8 @@ class LossyFrameEncoder {
       ctx_map[i] = i / lbuckets;
       // up to 3 contexts for chroma
       ctx_map[kNumOrders * num_dc_ctxs + i] =
-          num_dc_ctxs / lbuckets + (i % lbuckets);
-      ctx_map[2 * kNumOrders * num_dc_ctxs + i] =
-          num_dc_ctxs / lbuckets + (i % lbuckets);
+          ctx_map[2 * kNumOrders * num_dc_ctxs + i] =
+              num_dc_ctxs / lbuckets + (i % lbuckets);
     }
     enc_state_->shared.block_ctx_map.num_ctxs =
         *std::max_element(ctx_map.begin(), ctx_map.end()) + 1;
@@ -835,7 +835,7 @@ class LossyFrameEncoder {
     shared.frame_header.UpdateFlag(false, FrameHeader::kUseDcFrame);
     auto compute_dc_coeffs = [&](int group_index, int /* thread */) {
       modular_frame_encoder->AddVarDCTDC(dc, group_index, /*nl_dc=*/false,
-                                         enc_state_);
+                                         enc_state_, /*jpeg_transcode=*/true);
       modular_frame_encoder->AddACMetadata(group_index, /*jpeg_transcode=*/true,
                                            enc_state_);
     };
@@ -885,6 +885,7 @@ class LossyFrameEncoder {
     RunOnPool(pool_, 0, shared.frame_dim.num_groups, tokenize_group_init,
               tokenize_group, "TokenizeGroup");
     *frame_header = shared.frame_header;
+    doing_jpeg_recompression = true;
     return true;
   }
 
@@ -905,7 +906,7 @@ class LossyFrameEncoder {
                                               writer, kLayerDequantTables,
                                               aux_out_, modular_frame_encoder));
     if (enc_state_->cparams.speed_tier <= SpeedTier::kTortoise) {
-      ClusterGroups(enc_state_);
+      if (!doing_jpeg_recompression) ClusterGroups(enc_state_);
     }
     size_t num_histo_bits =
         CeilLog2Nonzero(enc_state_->shared.frame_dim.num_groups);
@@ -1003,6 +1004,7 @@ class LossyFrameEncoder {
   ThreadPool* pool_;
   AuxOut* aux_out_;
   std::vector<EncCache> group_caches_;
+  bool doing_jpeg_recompression = false;
 };
 
 Status EncodeFrame(const CompressParams& cparams_orig,
@@ -1030,6 +1032,7 @@ Status EncodeFrame(const CompressParams& cparams_orig,
   if (cparams.ec_resampling < cparams.resampling) {
     cparams.ec_resampling = cparams.resampling;
   }
+  if (cparams.resampling > 1) cparams.progressive_dc = 0;
 
   if (frame_info.dc_level + cparams.progressive_dc > 4) {
     return JXL_FAILURE("Too many levels of progressive DC");
@@ -1129,6 +1132,10 @@ Status EncodeFrame(const CompressParams& cparams_orig,
 
   const std::vector<ImageF>* extra_channels = &ib.extra_channels();
   std::vector<ImageF> extra_channels_storage;
+  // Clear patches
+  passes_enc_state->shared.image_features.patches = PatchDictionary();
+  passes_enc_state->shared.image_features.patches.SetPassesSharedState(
+      &passes_enc_state->shared);
 
   if (ib.IsJPEG()) {
     JXL_RETURN_IF_ERROR(lossy_frame_encoder.ComputeJPEGTranscodingData(
@@ -1162,6 +1169,7 @@ Status EncodeFrame(const CompressParams& cparams_orig,
     bool lossless = (frame_header->encoding == FrameEncoding::kModular &&
                      cparams.quality_pair.first == 100);
     if (ib.HasAlpha() && !ib.AlphaIsPremultiplied() &&
+        frame_header->frame_type == FrameType::kRegularFrame &&
         !ApplyOverride(cparams.keep_invisible, lossless) &&
         cparams.ec_resampling == cparams.resampling) {
       // simplify invisible pixels
