@@ -10,7 +10,6 @@
 #include <sys/types.h>
 
 #include <algorithm>
-#include <random>
 #include <string>
 #include <tuple>
 #include <utility>
@@ -19,6 +18,7 @@
 #include "lib/jxl/ans_params.h"
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/override.h"
+#include "lib/jxl/base/random.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/chroma_from_luma.h"
 #include "lib/jxl/color_management.h"
@@ -371,8 +371,7 @@ std::vector<PatchInfo> FindTextLikePatches(
   queue.clear();
 
   ImageF ccs;
-  std::mt19937 rng;
-  std::uniform_real_distribution<float> dist(0.5, 1.0);
+  Rng rng(0);
   bool paint_ccs = false;
   if (WantDebugOutput(aux_out)) {
     aux_out->DumpPlaneNormalized("is_background", is_background);
@@ -506,7 +505,7 @@ std::vector<PatchInfo> FindTextLikePatches(
         continue;
       }
       if (paint_ccs) {
-        float cc_color = dist(rng);
+        float cc_color = rng.UniformF(0.5, 1.0);
         for (std::pair<uint32_t, uint32_t> p : cc) {
           ccs.Row(p.second)[p.first] = cc_color;
         }
@@ -602,9 +601,6 @@ std::vector<PatchInfo> FindTextLikePatches(
 void FindBestPatchDictionary(const Image3F& opsin,
                              PassesEncoderState* JXL_RESTRICT state,
                              ThreadPool* pool, AuxOut* aux_out, bool is_xyb) {
-  state->shared.image_features.patches = PatchDictionary();
-  state->shared.image_features.patches.SetPassesSharedState(&state->shared);
-
   std::vector<PatchInfo> info =
       FindTextLikePatches(opsin, state, pool, aux_out, is_xyb);
 
@@ -752,10 +748,31 @@ void FindBestPatchDictionary(const Image3F& opsin,
   }
 
   CompressParams cparams = state->cparams;
-  cparams.resampling = 1;
-  cparams.ec_resampling = 1;
   // Recursive application of patches could create very weird issues.
   cparams.patches = Override::kOff;
+  // TODO(veluca): possibly change heuristics here.
+  if (!cparams.modular_mode) {
+    cparams.quality_pair.first = cparams.quality_pair.second =
+        90.f - cparams.butteraugli_distance * 5.f;
+  }
+
+  RoundtripPatchFrame(&reference_frame, state, 0, cparams, pool, true);
+
+  // TODO(veluca): this assumes that applying patches is commutative, which is
+  // not true for all blending modes. This code only produces kAdd patches, so
+  // this works out.
+  std::sort(positions.begin(), positions.end());
+  PatchDictionaryEncoder::SetPositions(&state->shared.image_features.patches,
+                                       std::move(positions));
+}
+
+void RoundtripPatchFrame(Image3F* reference_frame,
+                         PassesEncoderState* JXL_RESTRICT state, int idx,
+                         CompressParams& cparams, ThreadPool* pool,
+                         bool subtract) {
+  FrameInfo patch_frame_info;
+  cparams.resampling = 1;
+  cparams.ec_resampling = 1;
   cparams.dots = Override::kOff;
   cparams.noise = Override::kOff;
   cparams.modular_mode = true;
@@ -765,31 +782,19 @@ void FindBestPatchDictionary(const Image3F& opsin,
   cparams.qprogressive_mode = false;
   // Use gradient predictor and not Predictor::Best.
   cparams.options.predictor = Predictor::Gradient;
-  // TODO(veluca): possibly change heuristics here.
-  if (!cparams.modular_mode) {
-    cparams.quality_pair.first = cparams.quality_pair.second =
-        80 - cparams.butteraugli_distance * 12;
-  } else {
-    cparams.quality_pair.first = (100 + 3 * cparams.quality_pair.first) * 0.25f;
-    cparams.quality_pair.second =
-        (100 + 3 * cparams.quality_pair.second) * 0.25f;
-  }
-  FrameInfo patch_frame_info;
-  patch_frame_info.save_as_reference = 0;  // always saved.
+  patch_frame_info.save_as_reference = idx;  // always saved.
   patch_frame_info.frame_type = FrameType::kReferenceOnly;
   patch_frame_info.save_before_color_transform = true;
-
   ImageBundle ib(&state->shared.metadata->m);
   // TODO(veluca): metadata.color_encoding is a lie: ib is in XYB, but there is
   // no simple way to express that yet.
   patch_frame_info.ib_needs_color_transform = false;
-  patch_frame_info.save_as_reference = 0;
-  ib.SetFromImage(std::move(reference_frame),
+  ib.SetFromImage(std::move(*reference_frame),
                   state->shared.metadata->m.color_encoding);
   if (!ib.metadata()->extra_channel_info.empty()) {
-    // Add dummy extra channels to the patch image: patches do not yet support
-    // extra channels, but the codec expects that the amount of extra channels
-    // in frames matches that in the metadata of the codestream.
+    // Add dummy extra channels to the patch image: patch encoding does not yet
+    // support extra channels, but the codec expects that the amount of extra
+    // channels in frames matches that in the metadata of the codestream.
     std::vector<ImageF> extra_channels;
     extra_channels.reserve(ib.metadata()->extra_channel_info.size());
     for (size_t i = 0; i < ib.metadata()->extra_channel_info.size(); i++) {
@@ -797,18 +802,17 @@ void FindBestPatchDictionary(const Image3F& opsin,
       // Must initialize the image with data to not affect blending with
       // uninitialized memory.
       // TODO(lode): patches must copy and use the real extra channels instead.
-      FillImage(1.0f, &extra_channels.back());
+      ZeroFillImage(&extra_channels.back());
     }
     ib.SetExtraChannels(std::move(extra_channels));
   }
-
   PassesEncoderState roundtrip_state;
   auto special_frame = std::unique_ptr<BitWriter>(new BitWriter());
   JXL_CHECK(EncodeFrame(cparams, patch_frame_info, state->shared.metadata, ib,
                         &roundtrip_state, pool, special_frame.get(), nullptr));
   const Span<const uint8_t> encoded = special_frame->GetSpan();
   state->special_frames.emplace_back(std::move(special_frame));
-  if (cparams.butteraugli_distance < kMinButteraugliToSubtractOriginalPatches) {
+  if (subtract) {
     BitReader br(encoded);
     ImageBundle decoded(&state->shared.metadata->m);
     PassesDecoderState dec_state;
@@ -818,19 +822,20 @@ void FindBestPatchDictionary(const Image3F& opsin,
             state->shared.metadata->m.color_encoding.IsGray())));
     JXL_CHECK(DecodeFrame({}, &dec_state, pool, &br, &decoded,
                           *state->shared.metadata, /*constraints=*/nullptr));
+    // if the frame itself uses patches, we need to decode another frame
+    if (!dec_state.shared_storage.reference_frames[idx]
+             .storage.color()
+             ->xsize())
+      JXL_CHECK(DecodeFrame({}, &dec_state, pool, &br, &decoded,
+                            *state->shared.metadata, /*constraints=*/nullptr));
     JXL_CHECK(br.Close());
-    state->shared.reference_frames[0] =
-        std::move(dec_state.shared_storage.reference_frames[0]);
+    state->shared.reference_frames[idx] =
+        std::move(dec_state.shared_storage.reference_frames[idx]);
   } else {
-    state->shared.reference_frames[0].storage = std::move(ib);
+    state->shared.reference_frames[idx].storage = std::move(ib);
   }
-  state->shared.reference_frames[0].frame =
-      &state->shared.reference_frames[0].storage;
-  // TODO(veluca): this assumes that applying patches is commutative, which is
-  // not true for all blending modes. This code only produces kAdd patches, so
-  // this works out.
-  std::sort(positions.begin(), positions.end());
-  PatchDictionaryEncoder::SetPositions(&state->shared.image_features.patches,
-                                       std::move(positions));
+  state->shared.reference_frames[idx].frame =
+      &state->shared.reference_frames[idx].storage;
 }
+
 }  // namespace jxl
