@@ -19,6 +19,7 @@
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/override.h"
 #include "lib/jxl/base/padded_bytes.h"
+#include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/base/thread_pool_internal.h"
 #include "lib/jxl/codec_in_out.h"
 #include "lib/jxl/codec_y4m_testonly.h"
@@ -30,6 +31,7 @@
 #include "lib/jxl/enc_cache.h"
 #include "lib/jxl/enc_file.h"
 #include "lib/jxl/enc_params.h"
+#include "lib/jxl/fake_parallel_runner_testonly.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/image_bundle.h"
 #include "lib/jxl/image_ops.h"
@@ -133,7 +135,7 @@ TEST(JxlTest, RoundtripTinyFast) {
 
   CodecInOut io2;
   const size_t enc_bytes = Roundtrip(&io, cparams, dparams, pool, &io2);
-  printf("32x32 image size %zu bytes\n", enc_bytes);
+  printf("32x32 image size %" PRIuS " bytes\n", enc_bytes);
 }
 
 TEST(JxlTest, RoundtripSmallD1) {
@@ -154,9 +156,9 @@ TEST(JxlTest, RoundtripSmallD1) {
 
     compressed_size = Roundtrip(&io, cparams, dparams, pool, &io_out);
     EXPECT_LE(compressed_size, 1000u);
-    EXPECT_LE(ButteraugliDistance(io, io_out, cparams.ba_params,
-                                  /*distmap=*/nullptr, pool),
-              1.5);
+    EXPECT_THAT(ButteraugliDistance(io, io_out, cparams.ba_params,
+                                    /*distmap=*/nullptr, pool),
+                IsSlightlyBelow(1.0));
   }
 
   {
@@ -168,9 +170,9 @@ TEST(JxlTest, RoundtripSmallD1) {
     io_dim.ShrinkTo(io_dim.xsize() / 8, io_dim.ysize() / 8);
     EXPECT_LT(Roundtrip(&io_dim, cparams, dparams, pool, &io_out),
               compressed_size);
-    EXPECT_LE(ButteraugliDistance(io_dim, io_out, cparams.ba_params,
-                                  /*distmap=*/nullptr, pool),
-              1.5);
+    EXPECT_THAT(ButteraugliDistance(io_dim, io_out, cparams.ba_params,
+                                    /*distmap=*/nullptr, pool),
+                IsSlightlyBelow(1.5));
     EXPECT_EQ(io_dim.metadata.m.IntensityTarget(),
               io_out.metadata.m.IntensityTarget());
   }
@@ -194,18 +196,18 @@ TEST(JxlTest, RoundtripOtherTransforms) {
   const size_t compressed_size =
       Roundtrip(io.get(), cparams, dparams, pool, io2.get());
   EXPECT_LE(compressed_size, 23000u);
-  EXPECT_LE(ButteraugliDistance(*io, *io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            6);
+  EXPECT_THAT(ButteraugliDistance(*io, *io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(3.0));
 
   // Check the consistency when performing another roundtrip.
   std::unique_ptr<CodecInOut> io3 = jxl::make_unique<CodecInOut>();
   const size_t compressed_size2 =
       Roundtrip(io.get(), cparams, dparams, pool, io3.get());
   EXPECT_LE(compressed_size2, 23000u);
-  EXPECT_LE(ButteraugliDistance(*io, *io3, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            6);
+  EXPECT_THAT(ButteraugliDistance(*io, *io3, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(3.0));
 }
 
 TEST(JxlTest, RoundtripResample2) {
@@ -220,10 +222,11 @@ TEST(JxlTest, RoundtripResample2) {
   DecompressParams dparams;
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 17000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            12);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(8));
 }
+
 TEST(JxlTest, RoundtripResample2MT) {
   ThreadPoolInternal pool(4);
   const PaddedBytes orig =
@@ -247,6 +250,31 @@ TEST(JxlTest, RoundtripResample2MT) {
 #endif
 }
 
+// Roundtrip the image using a parallel runner that executes single-threaded but
+// in random order.
+TEST(JxlTest, RoundtripOutOfOrderProcessing) {
+  FakeParallelRunner fake_pool(/*order_seed=*/123, /*num_threads=*/8);
+  ThreadPool pool(&JxlFakeParallelRunner, &fake_pool);
+  const PaddedBytes orig =
+      ReadTestData("imagecompression.info/flower_foveon.png");
+  CodecInOut io;
+  ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, &pool));
+  // Image size is selected so that the block border needed is larger than the
+  // amount of pixels available on the next block.
+  io.ShrinkTo(513, 515);
+
+  CompressParams cparams;
+  // Force epf so we end up needing a lot of border.
+  cparams.epf = 3;
+
+  DecompressParams dparams;
+  CodecInOut io2;
+  Roundtrip(&io, cparams, dparams, &pool, &io2);
+
+  EXPECT_GE(1.5, ButteraugliDistance(io, io2, cparams.ba_params,
+                                     /*distmap=*/nullptr, &pool));
+}
+
 TEST(JxlTest, RoundtripResample4) {
   ThreadPool* pool = nullptr;
   const PaddedBytes orig =
@@ -259,9 +287,9 @@ TEST(JxlTest, RoundtripResample4) {
   DecompressParams dparams;
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 6000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            28);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(22));
 }
 
 TEST(JxlTest, RoundtripResample8) {
@@ -276,9 +304,9 @@ TEST(JxlTest, RoundtripResample8) {
   DecompressParams dparams;
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2100u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            80);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(50));
 }
 
 TEST(JxlTest, RoundtripUnalignedD2) {
@@ -295,9 +323,9 @@ TEST(JxlTest, RoundtripUnalignedD2) {
 
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 700u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            3.2);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.7));
 }
 
 #if JXL_TEST_NL
@@ -317,16 +345,16 @@ TEST(JxlTest, RoundtripMultiGroupNL) {
   cparams.butteraugli_distance = 1.0f;
   CodecInOut io2;
   Roundtrip(&io, cparams, dparams, &pool, &io2);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, &pool),
-            0.9f);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, &pool),
+              IsSlightlyBelow(0.9f));
 
   cparams.butteraugli_distance = 2.0f;
   CodecInOut io3;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 80000u);
-  EXPECT_LE(ButteraugliDistance(io, io3, cparams.ba_params,
-                                /*distmap=*/nullptr, &pool),
-            1.5f);
+  EXPECT_THAT(ButteraugliDistance(io, io3, cparams.ba_params,
+                                  /*distmap=*/nullptr, &pool),
+              IsSlightlyBelow(1.5f));
 }
 
 #endif
@@ -346,16 +374,16 @@ TEST(JxlTest, RoundtripMultiGroup) {
   cparams.speed_tier = SpeedTier::kKitten;
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 40000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, &pool),
-            1.99f);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, &pool),
+              IsSlightlyBelow(1.3f));
 
   cparams.butteraugli_distance = 2.0f;
   CodecInOut io3;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io3), 22100u);
-  EXPECT_LE(ButteraugliDistance(io, io3, cparams.ba_params,
-                                /*distmap=*/nullptr, &pool),
-            3.0f);
+  EXPECT_THAT(ButteraugliDistance(io, io3, cparams.ba_params,
+                                  /*distmap=*/nullptr, &pool),
+              IsSlightlyBelow(2.0f));
 }
 
 TEST(JxlTest, RoundtripLargeFast) {
@@ -474,9 +502,9 @@ TEST(JxlTest, RoundtripSmallNL) {
 
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 1500u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.7);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.7));
 }
 
 #endif
@@ -496,9 +524,9 @@ TEST(JxlTest, RoundtripNoGaborishNoAR) {
 
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 40000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            2.5);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(2.0));
 }
 
 TEST(JxlTest, RoundtripSmallNoGaborish) {
@@ -516,9 +544,9 @@ TEST(JxlTest, RoundtripSmallNoGaborish) {
 
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 900u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.7);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.2));
 }
 
 TEST(JxlTest, RoundtripSmallPatchesAlpha) {
@@ -548,9 +576,9 @@ TEST(JxlTest, RoundtripSmallPatchesAlpha) {
 
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            0.5f);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(0.22f));
 }
 
 TEST(JxlTest, RoundtripSmallPatches) {
@@ -576,9 +604,9 @@ TEST(JxlTest, RoundtripSmallPatches) {
 
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 2000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            0.5f);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(0.22f));
 }
 
 // Test header encoding of original bits per sample
@@ -683,9 +711,9 @@ TEST(JxlTest, RoundtripGrayscale) {
     EXPECT_TRUE(io2.Main().IsGray());
 
     EXPECT_LE(compressed.size(), 7000u);
-    EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                  /*distmap=*/nullptr, pool),
-              1.7777777);
+    EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                    /*distmap=*/nullptr, pool),
+                IsSlightlyBelow(1.6));
   }
 
   // Test with larger butteraugli distance and other settings enabled so
@@ -703,9 +731,9 @@ TEST(JxlTest, RoundtripGrayscale) {
     EXPECT_TRUE(io2.Main().IsGray());
 
     EXPECT_LE(compressed.size(), 1300u);
-    EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                  /*distmap=*/nullptr, pool),
-              9.0);
+    EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                    /*distmap=*/nullptr, pool),
+                IsSlightlyBelow(6.0));
   }
 }
 
@@ -738,9 +766,9 @@ TEST(JxlTest, RoundtripAlpha) {
 
   EXPECT_LE(compressed.size(), 10077u);
 
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.4);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.4));
 }
 
 TEST(JxlTest, RoundtripAlphaPremultiplied) {
@@ -772,13 +800,13 @@ TEST(JxlTest, RoundtripAlphaPremultiplied) {
 
   EXPECT_LE(compressed.size(), 10000u);
 
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.4);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.4));
   io2.Main().UnpremultiplyAlpha();
-  EXPECT_LE(ButteraugliDistance(io_nopremul, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.8);
+  EXPECT_THAT(ButteraugliDistance(io_nopremul, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.8));
 }
 
 TEST(JxlTest, RoundtripAlphaResampling) {
@@ -807,9 +835,9 @@ TEST(JxlTest, RoundtripAlphaResampling) {
 
   EXPECT_LE(compressed.size(), 15000u);
 
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            6.0);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(4.0));
 }
 
 TEST(JxlTest, RoundtripAlphaResamplingOnlyAlpha) {
@@ -837,9 +865,9 @@ TEST(JxlTest, RoundtripAlphaResamplingOnlyAlpha) {
 
   EXPECT_LE(compressed.size(), 31000u);
 
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.5);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.5));
 }
 
 TEST(JxlTest, RoundtripAlphaNonMultipleOf8) {
@@ -875,9 +903,9 @@ TEST(JxlTest, RoundtripAlphaNonMultipleOf8) {
   // the two contexts.
   // EXPECT_TRUE(SamePixels(io.Main().alpha(), io2.Main().alpha()));
   // TODO(robryk): Fix the distance estimate used in the encoder.
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            6.3);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(0.8));
 }
 
 TEST(JxlTest, RoundtripAlpha16) {
@@ -1199,9 +1227,9 @@ TEST(JxlTest, RoundtripYCbCr420) {
   EXPECT_LE(compressed.size(), 1320000u);
 
   // we're comparing an original PNG with a YCbCr 4:2:0 version
-  EXPECT_LE(ButteraugliDistance(io, io3, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            2.8);
+  EXPECT_THAT(ButteraugliDistance(io, io3, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(3.0));
 }
 
 TEST(JxlTest, RoundtripDots) {
@@ -1231,9 +1259,9 @@ TEST(JxlTest, RoundtripDots) {
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
   EXPECT_LE(compressed.size(), 400000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            2.2);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(0.3));
 }
 
 TEST(JxlTest, RoundtripNoise) {
@@ -1262,9 +1290,9 @@ TEST(JxlTest, RoundtripNoise) {
   EXPECT_TRUE(DecodeFile(dparams, compressed, &io2, pool));
 
   EXPECT_LE(compressed.size(), 40000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            2.2);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.6));
 }
 
 TEST(JxlTest, RoundtripLossless8Gray) {
@@ -1336,6 +1364,27 @@ TEST(JxlTest, RoundtripLosslessAnimation) {
             5e-4);
 }
 
+TEST(JxlTest, RoundtripAnimationPatches) {
+  ThreadPool* pool = nullptr;
+  const PaddedBytes orig = ReadTestData("jxl/animation_patches.gif");
+  CodecInOut io;
+  ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
+  ASSERT_EQ(2u, io.frames.size());
+
+  CompressParams cparams;
+  cparams.patches = Override::kOn;
+  DecompressParams dparams;
+  CodecInOut io2;
+  // 40k with no patches, 27k with patch frames encoded multiple times.
+  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 24000u);
+
+  EXPECT_EQ(io2.frames.size(), io.frames.size());
+  // >10 with broken patches
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.5));
+}
+
 #endif  // JPEGXL_ENABLE_GIF
 
 #if JPEGXL_ENABLE_JPEG
@@ -1624,30 +1673,9 @@ TEST(JxlTest, RoundtripProgressive) {
   cparams.progressive_mode = true;
   CodecInOut io2;
   EXPECT_LE(Roundtrip(&io, cparams, dparams, &pool, &io2), 40000u);
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, &pool),
-            4.0f);
-}
-
-TEST(JxlTest, RoundtripAnimationPatches) {
-  ThreadPool* pool = nullptr;
-  const PaddedBytes orig = ReadTestData("jxl/animation_patches.gif");
-  CodecInOut io;
-  ASSERT_TRUE(SetFromBytes(Span<const uint8_t>(orig), &io, pool));
-  ASSERT_EQ(2u, io.frames.size());
-
-  CompressParams cparams;
-  cparams.patches = Override::kOn;
-  DecompressParams dparams;
-  CodecInOut io2;
-  // 40k with no patches, 27k with patch frames encoded multiple times.
-  EXPECT_LE(Roundtrip(&io, cparams, dparams, pool, &io2), 24000u);
-
-  EXPECT_EQ(io2.frames.size(), io.frames.size());
-  // >10 with broken patches
-  EXPECT_LE(ButteraugliDistance(io, io2, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            2.0);
+  EXPECT_THAT(ButteraugliDistance(io, io2, cparams.ba_params,
+                                  /*distmap=*/nullptr, &pool),
+              IsSlightlyBelow(2.5f));
 }
 
 }  // namespace
