@@ -22,7 +22,7 @@ constexpr static int64_t kPredictionRound = ((1 << kPredExtraBits) >> 1) - 1;
 constexpr static size_t kNumProperties = 1;
 
 struct Header : public Fields {
-  const char *Name() const override { return "WeightedPredictorHeader"; }
+  JXL_FIELDS_NAME(WeightedPredictorHeader)
   // TODO(janwas): move to cc file, avoid including fields.h.
   Header() { Bundle::Init(this); }
 
@@ -86,7 +86,7 @@ struct State {
 
   // Approximates 4+(maxweight<<24)/(x+1), avoiding division
   JXL_INLINE uint32_t ErrorWeight(uint64_t x, uint32_t maxweight) const {
-    int shift = FloorLog2Nonzero(x + 1) - 5;
+    int shift = static_cast<int>(FloorLog2Nonzero(x + 1)) - 5;
     if (shift < 0) shift = 0;
     return 4 + ((maxweight * divlookup[x >> shift]) >> shift);
   }
@@ -398,46 +398,6 @@ struct PredictionResult {
   int32_t multiplier;
 };
 
-inline std::string PropertyName(size_t i) {
-  static_assert(kNumNonrefProperties == 16, "Update this function");
-  switch (i) {
-    case 0:
-      return "c";
-    case 1:
-      return "g";
-    case 2:
-      return "y";
-    case 3:
-      return "x";
-    case 4:
-      return "|N|";
-    case 5:
-      return "|W|";
-    case 6:
-      return "N";
-    case 7:
-      return "W";
-    case 8:
-      return "W-WW-NW+NWW";
-    case 9:
-      return "W+N-NW";
-    case 10:
-      return "W-NW";
-    case 11:
-      return "NW-N";
-    case 12:
-      return "N-NE";
-    case 13:
-      return "N-NN";
-    case 14:
-      return "W-WW";
-    case 15:
-      return "WGH";
-    default:
-      return "ch[" + ToString(15 - (int)i) + "]";
-  }
-}
-
 inline void InitPropsRow(
     Properties *p,
     const std::array<pixel_type, kNumStaticProperties> &static_props,
