@@ -7,7 +7,6 @@
 #include <stdio.h>
 
 #include <array>
-#include <random>
 #include <string>
 #include <utility>
 #include <vector>
@@ -126,9 +125,9 @@ TEST(ModularTest, RoundtripLossyDeltaPalette) {
   compressed_size = Roundtrip(&io, cparams, dparams, pool, &io_out);
   EXPECT_LE(compressed_size, 6800u);
   cparams.ba_params.intensity_target = 80.0f;
-  EXPECT_LE(ButteraugliDistance(io, io_out, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.5);
+  EXPECT_THAT(ButteraugliDistance(io, io_out, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.5));
 }
 TEST(ModularTest, RoundtripLossyDeltaPaletteWP) {
   ThreadPool* pool = nullptr;
@@ -153,9 +152,9 @@ TEST(ModularTest, RoundtripLossyDeltaPaletteWP) {
   compressed_size = Roundtrip(&io, cparams, dparams, pool, &io_out);
   EXPECT_LE(compressed_size, 7000u);
   cparams.ba_params.intensity_target = 80.0f;
-  EXPECT_LE(ButteraugliDistance(io, io_out, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            10.0);
+  EXPECT_THAT(ButteraugliDistance(io, io_out, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(10.0));
 }
 
 TEST(ModularTest, RoundtripLossy) {
@@ -176,9 +175,9 @@ TEST(ModularTest, RoundtripLossy) {
   compressed_size = Roundtrip(&io, cparams, dparams, pool, &io_out);
   EXPECT_LE(compressed_size, 40000u);
   cparams.ba_params.intensity_target = 80.0f;
-  EXPECT_LE(ButteraugliDistance(io, io_out, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            3.0);
+  EXPECT_THAT(ButteraugliDistance(io, io_out, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(2.0));
 }
 
 TEST(ModularTest, RoundtripLossy16) {
@@ -201,9 +200,9 @@ TEST(ModularTest, RoundtripLossy16) {
   compressed_size = Roundtrip(&io, cparams, dparams, pool, &io_out);
   EXPECT_LE(compressed_size, 400u);
   cparams.ba_params.intensity_target = 80.0f;
-  EXPECT_LE(ButteraugliDistance(io, io_out, cparams.ba_params,
-                                /*distmap=*/nullptr, pool),
-            1.5);
+  EXPECT_THAT(ButteraugliDistance(io, io_out, cparams.ba_params,
+                                  /*distmap=*/nullptr, pool),
+              IsSlightlyBelow(1.5));
 }
 
 TEST(ModularTest, RoundtripExtraProperties) {
@@ -212,12 +211,11 @@ TEST(ModularTest, RoundtripExtraProperties) {
   ModularOptions options;
   options.max_properties = 4;
   options.predictor = Predictor::Zero;
-  std::mt19937 rng(0);
-  std::uniform_int_distribution<> dist(0, 8);
+  Rng rng(0);
   for (size_t y = 0; y < kSize; y++) {
     for (size_t x = 0; x < kSize; x++) {
       image.channel[0].plane.Row(y)[x] = image.channel[2].plane.Row(y)[x] =
-          dist(rng);
+          rng.UniformU(0, 9);
     }
   }
   ZeroFillImage(&image.channel[1].plane);
