@@ -13,7 +13,6 @@
 #include <memory>
 #include <mutex>
 #include <numeric>
-#include <random>
 #include <string>
 #include <utility>
 #include <vector>
@@ -21,6 +20,7 @@
 #include "jxl/decode.h"
 #include "lib/extras/codec.h"
 #include "lib/extras/codec_png.h"
+#include "lib/extras/color_hints.h"
 #include "lib/extras/time.h"
 #include "lib/jxl/alpha.h"
 #include "lib/jxl/base/cache_aligned.h"
@@ -28,8 +28,9 @@
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/file_io.h"
 #include "lib/jxl/base/padded_bytes.h"
+#include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/base/profiler.h"
-#include "lib/jxl/base/robust_statistics.h"
+#include "lib/jxl/base/random.h"
 #include "lib/jxl/base/span.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/base/thread_pool_internal.h"
@@ -61,13 +62,14 @@ Status WritePNG(Image3F&& image, ThreadPool* pool,
   io.metadata.m.color_encoding = ColorEncoding::SRGB();
   io.SetFromImage(std::move(image), io.metadata.m.color_encoding);
   PaddedBytes compressed;
-  JXL_CHECK(EncodeImagePNG(&io, io.Main().c_current(), 8, pool, &compressed));
+  JXL_CHECK(
+      extras::EncodeImagePNG(&io, io.Main().c_current(), 8, pool, &compressed));
   return WriteFile(compressed, filename);
 }
 
 Status ReadPNG(const std::string& filename, Image3F* image) {
   CodecInOut io;
-  JXL_CHECK(SetFromFile(filename, &io));
+  JXL_CHECK(SetFromFile(filename, ColorHints(), &io));
   *image = CopyImage(*io.Main().color());
   return true;
 }
@@ -176,7 +178,8 @@ void DoCompress(const std::string& filename, const CodecInOut& io,
     if (!Args()->silent_errors) {
       // Animated gifs not supported yet?
       fprintf(stderr,
-              "Frame sizes not equal, is this an animated gif? %s %s %zu %zu\n",
+              "Frame sizes not equal, is this an animated gif? %s %s %" PRIuS
+              " %" PRIuS "\n",
               codec_name.c_str(), name.c_str(), io.frames.size(),
               io2.frames.size());
     }
@@ -479,15 +482,19 @@ void WriteHtmlReport(const std::string& codec_desc,
       url_out = Base64Image(fname_out);
       url_heatmap = Base64Image(fname_heatmap);
     }
-    std::string number = StringPrintf("%zu", i);
+    std::string number = StringPrintf("%" PRIuS, i);
     const CodecInOut& image = *images[i];
     size_t xsize = image.frames.size() == 1 ? image.xsize() : 0;
     size_t ysize = image.frames.size() == 1 ? image.ysize() : 0;
-    std::string html_width = StringPrintf("%zupx", xsize);
-    std::string html_height = StringPrintf("%zupx", ysize);
+    std::string html_width = StringPrintf("%" PRIuS "px", xsize);
+    std::string html_height = StringPrintf("%" PRIuS "px", ysize);
     double bpp = tasks[i]->stats.total_compressed_size * 8.0 /
                  tasks[i]->stats.total_input_pixels;
-    std::string compressed_title = StringPrintf("compressed: %f bpp", bpp);
+    double pnorm =
+        tasks[i]->stats.distance_p_norm / tasks[i]->stats.total_input_pixels;
+    double max_dist = tasks[i]->stats.max_distance;
+    std::string compressed_title = StringPrintf(
+        "compressed. bpp: %f, pnorm: %f, max dist: %f", bpp, pnorm, max_dist);
     out_html += "<div onclick=\"toggle3(" + number +
                 ");\" style=\"display:inline-block;width:" + html_width +
                 ";height:" + html_height +
@@ -598,10 +605,10 @@ struct StatPrinter {
 
     const double rmse =
         std::sqrt(t.stats.distance_2 / t.stats.total_input_pixels);
-    const double psnr =
-        t.stats.total_compressed_size == 0
-            ? 0.0
-            : (t.stats.distance_2 == 0) ? 99.99 : (20 * std::log10(1 / rmse));
+    const double psnr = t.stats.total_compressed_size == 0 ? 0.0
+                        : (t.stats.distance_2 == 0)
+                            ? 99.99
+                            : (20 * std::log10(1 / rmse));
     size_t pixels = t.stats.total_input_pixels;
 
     const double enc_mps =
@@ -609,7 +616,8 @@ struct StatPrinter {
     const double dec_mps =
         t.stats.total_input_pixels / (1000000.0 * t.stats.total_time_decode);
     if (Args()->print_details_csv) {
-      printf("%s,%s,%zd,%zd,%zd,%.8f,%.8f,%.8f,%.8f,%.8f,%.8f,%.8f,%.8f",
+      printf("%s,%s,%" PRIdS ",%" PRIdS ",%" PRIdS
+             ",%.8f,%.8f,%.8f,%.8f,%.8f,%.8f,%.8f,%.8f",
              (*methods_)[t.idx_method].c_str(),
              FileBaseName((*fnames_)[t.idx_image]).c_str(),
              t.stats.total_errors, t.stats.total_compressed_size, pixels,
@@ -631,8 +639,8 @@ struct StatPrinter {
         printf(" ");
       }
       printf(
-          "error:%zd    size:%8zd    pixels:%9zd    enc_speed:%8.8f"
-          "    dec_speed:%8.8f    bpp:%10.8f    dist:%10.8f"
+          "error:%" PRIdS "    size:%8" PRIdS "    pixels:%9" PRIdS
+          "    enc_speed:%8.8f    dec_speed:%8.8f    bpp:%10.8f    dist:%10.8f"
           "    psnr:%10.8f    p:%10.8f    bppp:%10.8f    qabpp:%10.8f ",
           t.stats.total_errors, t.stats.total_compressed_size, pixels, enc_mps,
           dec_mps, comp_bpp, t.stats.max_distance, psnr, p_norm, bpp_p_norm,
@@ -663,7 +671,7 @@ struct StatPrinter {
     std::string out;
 
     method_stats.PrintMoreStats();  // not concurrent
-    out += method_stats.PrintLine(method, fnames_->size(), /*num_threads=*/1);
+    out += method_stats.PrintLine(method, fnames_->size());
 
     if (Args()->write_html_report) {
       WriteHtmlReport(method, *fnames_, tasks, images,
@@ -671,7 +679,7 @@ struct StatPrinter {
     }
 
     stats_aggregate_.push_back(
-        method_stats.ComputeColumns(method, fnames_->size(), 1));
+        method_stats.ComputeColumns(method, fnames_->size()));
 
     printf("%s", out.c_str());
     fflush(stdout);
@@ -686,12 +694,14 @@ struct StatPrinter {
       printf("```\n");
     }
     if (fnames_->size() == 1) printf("%s\n", (*fnames_)[0].c_str());
-    printf("%s", PrintHeader().c_str());
+    printf("%s", PrintHeader(*extra_metrics_names_).c_str());
     fflush(stdout);
   }
 
   void PrintStatsFooter() {
-    printf("%s", PrintAggregate(stats_aggregate_).c_str());
+    printf(
+        "%s",
+        PrintAggregate(extra_metrics_names_->size(), stats_aggregate_).c_str());
     if (Args()->markdown) printf("```\n");
     printf("\n");
     fflush(stdout);
@@ -816,10 +826,11 @@ class Benchmark {
     pool->RunOnEachThread([&](int /*task*/, const int thread) {
       const size_t index = *next_index + static_cast<size_t>(thread);
       if (index < cpus.size()) {
-        // printf("pin pool %p thread %3d to index %3zu = cpu %3d\n",
+        // printf("pin pool %p thread %3d to index %3" PRIuS " = cpu %3d\n",
         //        static_cast<void*>(pool), thread, index, cpus[index]);
         if (!jpegxl::tools::cpu::PinThreadToCPU(cpus[index])) {
-          fprintf(stderr, "WARNING: failed to pin thread %d, next %zu.\n",
+          fprintf(stderr,
+                  "WARNING: failed to pin thread %d, next %" PRIuS ".\n",
                   thread, *next_index);
         }
       }
@@ -897,7 +908,8 @@ class Benchmark {
                                    const std::string& sample_tmp_dir,
                                    int num_samples, size_t size) {
     JXL_CHECK(!sample_tmp_dir.empty());
-    fprintf(stderr, "Creating samples of %zux%zu tiles...\n", size, size);
+    fprintf(stderr, "Creating samples of %" PRIuS "x%" PRIuS " tiles...\n",
+            size, size);
     StringVec fnames_out;
     std::vector<Image3F> images;
     std::vector<size_t> offsets;
@@ -912,15 +924,15 @@ class Benchmark {
       images.emplace_back(std::move(img));
     }
     JXL_CHECK(MakeDir(sample_tmp_dir));
-    std::mt19937_64 rng;
+    Rng rng(0);
     for (int i = 0; i < num_samples; ++i) {
-      int val = std::uniform_int_distribution<>(0, offsets.back())(rng);
+      int val = rng.UniformI(0, offsets.back());
       size_t idx = (std::lower_bound(offsets.begin(), offsets.end(), val) -
                     offsets.begin());
       JXL_CHECK(idx < images.size());
       const Image3F& img = images[idx];
-      int x0 = std::uniform_int_distribution<>(0, img.xsize() - size)(rng);
-      int y0 = std::uniform_int_distribution<>(0, img.ysize() - size)(rng);
+      int x0 = rng.UniformI(0, img.xsize() - size);
+      int y0 = rng.UniformI(0, img.ysize() - size);
       Image3F sample(size, size);
       for (size_t c = 0; c < 3; ++c) {
         for (size_t y = 0; y < size; ++y) {
@@ -971,12 +983,11 @@ class Benchmark {
           Status ok = true;
 
           loaded_images[i].target_nits = Args()->intensity_target;
-          loaded_images[i].dec_hints = Args()->dec_hints;
           loaded_images[i].dec_target = jpeg_transcoding_requested
                                             ? DecodeTarget::kQuantizedCoeffs
                                             : DecodeTarget::kPixels;
           if (!Args()->decode_only) {
-            ok = SetFromFile(fnames[i], &loaded_images[i]);
+            ok = SetFromFile(fnames[i], Args()->color_hints, &loaded_images[i]);
           }
           if (!ok) {
             if (!Args()->silent_errors) {
