@@ -25,6 +25,7 @@
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/padded_bytes.h"
+#include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/base/profiler.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/base/thread_pool_internal.h"
@@ -60,6 +61,9 @@ static inline bool ParseColorTransform(const char* arg,
 static inline bool ParseIntensityTarget(const char* arg, float* out) {
   return ParseFloat(arg, out) && *out > 0;
 }
+static inline bool ParsePhotonNoiseParameter(const char* arg, float* out) {
+  return strncmp(arg, "ISO", 3) == 0 && ParseFloat(arg + 3, out) && *out > 0;
+}
 
 // Proposes a distance to try for a given bpp target. This could depend
 // on the entropy in the image, too, but let's start with something.
@@ -70,7 +74,7 @@ static double ApproximateDistanceForBPP(double bpp) {
 jxl::Status LoadSaliencyMap(const std::string& filename_heatmap,
                             jxl::ThreadPool* pool, jxl::ImageF* out_map) {
   jxl::CodecInOut io_heatmap;
-  if (!SetFromFile(filename_heatmap, &io_heatmap, pool)) {
+  if (!SetFromFile(filename_heatmap, jxl::ColorHints(), &io_heatmap, pool)) {
     return JXL_FAILURE("Could not load heatmap.");
   }
   *out_map = std::move(io_heatmap.Main().color()->Plane(0));
@@ -116,7 +120,7 @@ void SetModularQualityForBitrate(jxl::ThreadPoolInternal* pool,
           quality);
       break;
     }
-    printf("Quality %.2f yields %6zu bytes, %.3f bpp.\n", quality,
+    printf("Quality %.2f yields %6" PRIuS " bytes, %.3f bpp.\n", quality,
            candidate.size(), candidate.size() * 8.0 / pixels);
     const double ratio = static_cast<double>(candidate.size()) / target_size;
     const double loss = std::abs(1.0 - ratio);
@@ -191,8 +195,8 @@ void SetParametersForSizeOrBitrate(jxl::ThreadPoolInternal* pool,
           best_dist);
       break;
     }
-    printf("Butteraugli distance %.3f yields %6zu bytes, %.3f bpp.\n", dist,
-           candidate.size(), candidate.size() * 8.0 / pixels);
+    printf("Butteraugli distance %.3f yields %6" PRIuS " bytes, %.3f bpp.\n",
+           dist, candidate.size(), candidate.size() * 8.0 / pixels);
     const double ratio = static_cast<double>(candidate.size()) / target_size;
     const double loss = std::max(ratio, 1.0 / std::max(ratio, 1e-30));
     if (best_loss > loss) {
@@ -245,7 +249,8 @@ void PrintMode(jxl::ThreadPoolInternal* pool, const jxl::CodecInOut& io,
   const char* speed = SpeedTierName(args.params.speed_tier);
   const std::string quality = QualityFromArgs(args);
   fprintf(stderr,
-          "Read %zux%zu image, %.1f MP/s\n"
+          "Read %" PRIuS "x%" PRIuS
+          " image, %.1f MP/s\n"
           "Encoding [%s%s, %s, %s",
           io.xsize(), io.ysize(), decode_mps,
           (args.use_container ? "Container | " : ""), mode, quality.c_str(),
@@ -253,13 +258,13 @@ void PrintMode(jxl::ThreadPoolInternal* pool, const jxl::CodecInOut& io,
   if (args.use_container) {
     if (args.jpeg_transcode) fprintf(stderr, " | JPEG reconstruction data");
     if (!io.blobs.exif.empty())
-      fprintf(stderr, " | %zu-byte Exif", io.blobs.exif.size());
+      fprintf(stderr, " | %" PRIuS "-byte Exif", io.blobs.exif.size());
     if (!io.blobs.xmp.empty())
-      fprintf(stderr, " | %zu-byte XMP", io.blobs.xmp.size());
+      fprintf(stderr, " | %" PRIuS "-byte XMP", io.blobs.xmp.size());
     if (!io.blobs.jumbf.empty())
-      fprintf(stderr, " | %zu-byte JUMBF", io.blobs.jumbf.size());
+      fprintf(stderr, " | %" PRIuS "-byte JUMBF", io.blobs.jumbf.size());
   }
-  fprintf(stderr, "], %zu threads.\n", pool->NumWorkerThreads());
+  fprintf(stderr, "], %" PRIuS " threads.\n", pool->NumWorkerThreads());
 }
 
 }  // namespace
@@ -304,7 +309,7 @@ void CompressArgs::AddCommandLineOptions(CommandLineParser* cmdline) {
   // Target distance/size/bpp
   opt_distance_id = cmdline->AddOptionValue(
       'd', "distance", "maxError",
-      ("Max. butteraugli distance, lower = higher quality. Range: 0 .. 15.\n"
+      ("Max. butteraugli distance, lower = higher quality. Range: 0 .. 25.\n"
        "    0.0 = mathematically lossless. Default for already-lossy input "
        "(JPEG/GIF).\n"
        "    1.0 = visually lossless. Default for other input.\n"
@@ -394,6 +399,10 @@ void CompressArgs::AddCommandLineOptions(CommandLineParser* cmdline) {
                          "Do lossy transcode of input JPEG file (decode to "
                          "pixels instead of doing lossless transcode).",
                          &jpeg_transcode, &SetBooleanFalse, 1);
+  cmdline->AddOptionFlag('\0', "jpeg_transcode_disable_cfl",
+                         "Disable CFL for lossless JPEG recompression",
+                         &params.force_cfl_jpeg_recompression, &SetBooleanFalse,
+                         2);
 
   opt_num_threads_id = cmdline->AddOptionValue(
       '\0', "num_threads", "N", "number of worker threads (zero = none).",
@@ -404,15 +413,24 @@ void CompressArgs::AddCommandLineOptions(CommandLineParser* cmdline) {
   cmdline->AddOptionValue('\0', "noise", "0|1",
                           "force disable/enable noise generation.",
                           &params.noise, &ParseOverride, 1);
+  cmdline->AddOptionValue(
+      '\0', "photon_noise", "ISO3200",
+      "Set the noise to approximately what it would be at a given nominal "
+      "exposure on a 35mm camera. For formats other than 35mm, or when the "
+      "whole sensor was not used, you can multiply the ISO value by the "
+      "equivalence ratio squared, for example by 2.25 for an APS-C camera.",
+      &params.photon_noise_iso, &ParsePhotonNoiseParameter, 1);
   cmdline->AddOptionValue('\0', "dots", "0|1",
                           "force disable/enable dots generation.", &params.dots,
                           &ParseOverride, 1);
   cmdline->AddOptionValue('\0', "patches", "0|1",
                           "force disable/enable patches generation.",
                           &params.patches, &ParseOverride, 1);
-  cmdline->AddOptionValue('\0', "resampling", "1|2|4|8",
-                          "Subsample all color channels by this factor.",
-                          &params.resampling, &ParseUnsigned, 1);
+  cmdline->AddOptionValue(
+      '\0', "resampling", "0|1|2|4|8",
+      "Subsample all color channels by this factor, or use 0 to choose the "
+      "resampling factor based on distance.",
+      &params.resampling, &ParseUnsigned, 0);
   cmdline->AddOptionValue(
       '\0', "ec_resampling", "1|2|4|8",
       "Subsample all extra channels by this factor. If this value is smaller "
@@ -452,7 +470,7 @@ void CompressArgs::AddCommandLineOptions(CommandLineParser* cmdline) {
       'x', "dec-hints", "key=value",
       "color_space indicates the ColorEncoding, see Description();\n"
       "icc_pathname refers to a binary file containing an ICC profile.",
-      &dec_hints, &ParseAndAppendKeyValue, 1);
+      &color_hints, &ParseAndAppendKeyValue, 1);
 
   cmdline->AddOptionValue(
       '\0', "override_bitdepth", "0=use from image, 1-32=override",
@@ -607,7 +625,7 @@ jxl::Status CompressArgs::ValidateArgs(const CommandLineParser& cmdline) {
 
   if (got_distance) {
     constexpr float butteraugli_min_dist = 0.1f;
-    constexpr float butteraugli_max_dist = 15.0f;
+    constexpr float butteraugli_max_dist = 25.0f;
     if (!(0 <= params.butteraugli_distance &&
           params.butteraugli_distance <= butteraugli_max_dist)) {
       fprintf(stderr, "Invalid/out of range distance, try 0 to %g.\n",
@@ -624,6 +642,10 @@ jxl::Status CompressArgs::ValidateArgs(const CommandLineParser& cmdline) {
     default_settings = false;
   }
 
+  if (got_target_bpp || got_target_size) {
+    jpeg_transcode = false;
+  }
+
   if (got_target_bpp + got_target_size + got_distance + got_quality > 1) {
     fprintf(stderr,
             "You can specify only one of '--distance', '-q', "
@@ -727,11 +749,11 @@ jxl::Status LoadAll(CompressArgs& args, jxl::ThreadPoolInternal* pool,
   const double t0 = jxl::Now();
 
   io->target_nits = args.intensity_target;
-  io->dec_hints = args.dec_hints;
   io->dec_target = (args.jpeg_transcode ? jxl::DecodeTarget::kQuantizedCoeffs
                                         : jxl::DecodeTarget::kPixels);
   jxl::Codec input_codec;
-  if (!SetFromFile(args.params.file_in, io, nullptr, &input_codec)) {
+  if (!SetFromFile(args.params.file_in, args.color_hints, io, nullptr,
+                   &input_codec)) {
     fprintf(stderr, "Failed to read image %s.\n", args.params.file_in);
     return false;
   }
@@ -742,12 +764,6 @@ jxl::Status LoadAll(CompressArgs& args, jxl::ThreadPoolInternal* pool,
     args.params.modular_mode = true;
     args.params.quality_pair.first = args.params.quality_pair.second = 100;
   }
-  if (args.params.modular_mode && args.params.quality_pair.first < 100) {
-    if (io->metadata.m.bit_depth.floating_point_sample) {
-      // for lossy modular, pretend pfm/exr is integer data
-      io->metadata.m.SetUintSamples(12);
-    }
-  }
   if (args.override_bitdepth != 0) {
     if (args.override_bitdepth == 32) {
       io->metadata.m.SetFloat32Samples();
@@ -825,7 +841,7 @@ jxl::Status CompressJxl(jxl::CodecInOut& io, double decode_mps,
   if (print_stats) {
     const double bpp =
         static_cast<double>(compressed->size() * jxl::kBitsPerByte) / pixels;
-    fprintf(stderr, "Compressed to %zu bytes (%.3f bpp%s).\n",
+    fprintf(stderr, "Compressed to %" PRIuS " bytes (%.3f bpp%s).\n",
             compressed->size(), bpp / io.frames.size(),
             io.frames.size() == 1 ? "" : "/frame");
     JXL_CHECK(stats.Print(args.num_threads));
