@@ -43,7 +43,9 @@ struct FuzzSpec {
   // Whether to use the callback mechanism for the output image or not.
   bool use_callback;
   bool keep_orientation;
-  uint32_t streaming_seed;
+  bool decode_boxes;
+  // Used for random variation of chunk sizes, extra channels, ... to get
+  uint32_t random_seed;
 };
 
 template <typename It>
@@ -76,8 +78,8 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
       std::min<size_t>(2, JxlThreadParallelRunnerDefaultNumWorkerThreads());
   auto runner = JxlThreadParallelRunnerMake(nullptr, num_threads);
 
-  std::mt19937 mt(spec.streaming_seed);
-  std::exponential_distribution<> dis(kStreamingTargetNumberOfChunks);
+  std::mt19937 mt(spec.random_seed);
+  std::exponential_distribution<> dis_streaming(kStreamingTargetNumberOfChunks);
 
   auto dec = JxlDecoderMake(nullptr);
   if (JXL_DEC_SUCCESS !=
@@ -85,7 +87,7 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
           dec.get(), JXL_DEC_BASIC_INFO | JXL_DEC_EXTENSIONS |
                          JXL_DEC_COLOR_ENCODING | JXL_DEC_PREVIEW_IMAGE |
                          JXL_DEC_FRAME | JXL_DEC_FULL_IMAGE |
-                         JXL_DEC_JPEG_RECONSTRUCTION)) {
+                         JXL_DEC_JPEG_RECONSTRUCTION | JXL_DEC_BOX)) {
     return false;
   }
   if (JXL_DEC_SUCCESS != JxlDecoderSetParallelRunner(dec.get(),
@@ -105,6 +107,7 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
   if (!spec.use_streaming) {
     // Set all input at once
     JxlDecoderSetInput(dec.get(), jxl, size);
+    JxlDecoderCloseInput(dec.get());
   }
 
   bool seen_basic_info = false;
@@ -130,6 +133,8 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
   bool want_preview = false;
   std::vector<uint8_t> preview_pixels;
 
+  std::vector<uint8_t> extra_channel_pixels;
+
   // Callback function used when decoding with use_callback.
   struct DecodeCallbackData {
     JxlBasicInfo info;
@@ -159,6 +164,15 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
     }
   };
 
+  JxlExtraChannelInfo extra_channel_info;
+
+  std::vector<uint8_t> box_buffer;
+
+  if (spec.decode_boxes &&
+      JXL_DEC_SUCCESS != JxlDecoderSetDecompressBoxes(dec.get(), JXL_TRUE)) {
+    // error ignored, can still fuzz if it doesn't brotli-decompress brob boxes.
+  }
+
   for (;;) {
     JxlDecoderStatus status = JxlDecoderProcessInput(dec.get());
     if (status == JXL_DEC_ERROR) {
@@ -171,8 +185,8 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
         jxl += used;
         leftover -= used;
         streaming_size -= used;
-        size_t chunk_size =
-            std::max<size_t>(1, size * std::min<double>(1.0, dis(mt)));
+        size_t chunk_size = std::max<size_t>(
+            1, size * std::min<double>(1.0, dis_streaming(mt)));
         size_t add_size =
             std::min<size_t>(chunk_size, leftover - streaming_size);
         if (add_size == 0) {
@@ -180,7 +194,14 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
           return false;
         }
         streaming_size += add_size;
-        JxlDecoderSetInput(dec.get(), jxl, streaming_size);
+        if (JXL_DEC_SUCCESS !=
+            JxlDecoderSetInput(dec.get(), jxl, streaming_size)) {
+          return false;
+        }
+        if (leftover == streaming_size) {
+          // All possible input bytes given
+          JxlDecoderCloseInput(dec.get());
+        }
 
         if (!tested_flush && seen_frame) {
           // Test flush max once to avoid too slow fuzzer run
@@ -239,14 +260,13 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
       }
 
       for (size_t ec = 0; ec < info.num_extra_channels; ++ec) {
-        JxlExtraChannelInfo ec_info;
-        memset(&ec_info, 0, sizeof(ec_info));
+        memset(&extra_channel_info, 0, sizeof(extra_channel_info));
         if (JXL_DEC_SUCCESS !=
-            JxlDecoderGetExtraChannelInfo(dec.get(), ec, &ec_info)) {
+            JxlDecoderGetExtraChannelInfo(dec.get(), ec, &extra_channel_info)) {
           abort();
         }
-        Consume(ec_info);
-        std::vector<char> ec_name(ec_info.name_length + 1);
+        Consume(extra_channel_info);
+        std::vector<char> ec_name(extra_channel_info.name_length + 1);
         if (JXL_DEC_SUCCESS != JxlDecoderGetExtraChannelName(dec.get(), ec,
                                                              ec_name.data(),
                                                              ec_name.size())) {
@@ -334,6 +354,27 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
         seen_need_image_out = true;
       }
 
+      if (info.num_extra_channels > 0) {
+        std::uniform_int_distribution<> dis(0, info.num_extra_channels);
+        size_t ec_index = dis(mt);
+        // There is also a probability no extra channel is chosen
+        if (ec_index < info.num_extra_channels) {
+          size_t ec_index = info.num_extra_channels - 1;
+          size_t ec_size;
+          if (JXL_DEC_SUCCESS != JxlDecoderExtraChannelBufferSize(
+                                     dec.get(), &format, &ec_size, ec_index)) {
+            return false;
+          }
+          extra_channel_pixels.resize(ec_size);
+          if (JXL_DEC_SUCCESS !=
+              JxlDecoderSetExtraChannelBuffer(dec.get(), &format,
+                                              extra_channel_pixels.data(),
+                                              ec_size, ec_index)) {
+            return false;
+          }
+        }
+      }
+
       if (spec.use_callback) {
         if (JXL_DEC_SUCCESS !=
             JxlDecoderSetImageOutCallback(dec.get(), &format, decode_callback,
@@ -422,6 +463,28 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
       // It's not required to call JxlDecoderReleaseInput(dec.get()) here since
       // the decoder will be destroyed.
       return true;
+    } else if (status == JXL_DEC_BOX) {
+      if (spec.decode_boxes) {
+        if (!box_buffer.empty()) {
+          size_t remaining = JxlDecoderReleaseBoxBuffer(dec.get());
+          size_t box_size = box_buffer.size() - remaining;
+          if (box_size != 0) {
+            Consume(box_buffer.begin(), box_buffer.begin() + box_size);
+            box_buffer.clear();
+          }
+        }
+        box_buffer.resize(64);
+        JxlDecoderSetBoxBuffer(dec.get(), box_buffer.data(), box_buffer.size());
+      }
+    } else if (status == JXL_DEC_BOX_NEED_MORE_OUTPUT) {
+      if (!spec.decode_boxes) {
+        abort();  // Not expected when not setting output buffer
+      }
+      size_t remaining = JxlDecoderReleaseBoxBuffer(dec.get());
+      size_t box_size = box_buffer.size() - remaining;
+      box_buffer.resize(box_buffer.size() * 2);
+      JxlDecoderSetBoxBuffer(dec.get(), box_buffer.data() + box_size,
+                             box_buffer.size() - box_size);
     } else {
       return false;
     }
@@ -450,7 +513,7 @@ int TestOneInput(const uint8_t* data, size_t size) {
   FuzzSpec spec;
   // Allows some different possible variations in the chunk sizes of the
   // streaming case
-  spec.streaming_seed = flags ^ size;
+  spec.random_seed = flags ^ size;
   spec.get_alpha = !!getFlag(1);
   spec.get_grayscale = !!getFlag(1);
   spec.use_streaming = !!getFlag(1);
@@ -460,6 +523,7 @@ int TestOneInput(const uint8_t* data, size_t size) {
   spec.output_type = static_cast<JxlDataType>(getFlag(JXL_TYPE_FLOAT16));
   spec.output_endianness = static_cast<JxlEndianness>(getFlag(JXL_BIG_ENDIAN));
   spec.output_align = getFlag(16);
+  spec.decode_boxes = !!getFlag(1);
 
   std::vector<uint8_t> pixels;
   std::vector<uint8_t> jpeg;
@@ -468,7 +532,7 @@ int TestOneInput(const uint8_t* data, size_t size) {
   size_t max_pixels = 1 << 21;
 
   const auto targets = hwy::SupportedAndGeneratedTargets();
-  hwy::SetSupportedTargetsForTest(getFlag(targets.size() - 1));
+  hwy::SetSupportedTargetsForTest(targets[getFlag(targets.size() - 1)]);
   DecodeJpegXl(data, size, max_pixels, spec, &pixels, &jpeg, &xsize, &ysize,
                &icc);
   hwy::SetSupportedTargetsForTest(0);
